@@ -3,13 +3,53 @@
 import argparse
 import logging
 import sys
+from typing import List, Tuple
 
 from .actions import Actions
-from .config import Config
+from .carddav_object import CarddavObject
+from .config import Config, ConfigError
+from .query import AndQuery, AnyQuery, FieldQuery, NameQuery, parse
 from .version import version as khard_version
 
 
-def create_parsers():
+logger = logging.getLogger(__name__)
+
+
+class FieldsArgument:
+    """A factory to create callable objects for add_argument's type= parameter.
+
+    The object can parse comma seperated strings into list of strings, and can
+    also check if the single elements are spelled correctly.
+    """
+
+    def __init__(self, *choices: str, nested: bool = False) -> None:
+        """Initialize the factory
+
+        :param choices: the comma seperated strings must be one of these
+        :param nested: if this is true the comma seperated strings may
+            designate nested fields and only the first component (seperated by
+            a dot) must match on of the choices
+        """
+        self._choices = sorted(choices)
+        self._nested = nested
+
+    def __call__(self, argument: str) -> List[str]:
+        ret = []
+        for candidate in argument.split(","):
+            candidate = candidate.lower()
+            test = candidate.split('.')[0] if self._nested else candidate
+            if test in self._choices:
+                ret.append(candidate)
+            else:
+                choices = ', '.join('"{}"'.format(c) for c in self._choices)
+                raise argparse.ArgumentTypeError(
+                    '"{}" is not an accepted field. Accepted fields are {}.'
+                    .format(test, choices))
+        return ret
+
+
+def create_parsers() -> Tuple[argparse.ArgumentParser,
+                              argparse.ArgumentParser]:
     """Create two argument parsers.
 
     The first parser is manly used to find the config file which can than be
@@ -18,7 +58,6 @@ def create_parsers():
     further options and arguments.
 
     :returns: the two parsers for the first and the second parsing pass
-    :rtype: (argparse.ArgumentParser, argparse.ArgumentParser)
     """
     # Create the base argument parser.  It will be reused for the first and
     # second round of argument parsing.
@@ -31,7 +70,7 @@ def create_parsers():
     base.add_argument("--skip-unparsable", action="store_true",
                       help="skip unparsable vcard files")
     base.add_argument("-v", "--version", action="version",
-                      version="Khard version %s" % khard_version)
+                      version="Khard version {}".format(khard_version))
 
     # Create the first argument parser.  Its main job is to set the correct
     # config file.  The config file is needed to get the default command if no
@@ -118,12 +157,14 @@ def create_parsers():
         "to incomplete results.")
     default_search_parser.add_argument(
         "-e", "--strict-search", action="store_true",
-        help="narrow contact search to name field")
+        help="DEPRECATED use the new query syntax instead")
     default_search_parser.add_argument(
-        "-u", "--uid", default="", help="select contact by uid")
+        "-u", "--uid", type=lambda x: FieldQuery("uid", x),
+        help="DEPRECATED use the new query syntax instead")
     default_search_parser.add_argument(
-        "search_terms", nargs="*", metavar="search terms",
-        help="search in all fields to find matching contact")
+        "search_terms", nargs="*", metavar="search terms", type=parse,
+        default=[], help="search in specified or all fields to find matching "
+        "contact")
     merge_search_parser = argparse.ArgumentParser(add_help=False)
     merge_search_parser.add_argument(
         "-f", "--search-in-source-files", action="store_true",
@@ -132,20 +173,24 @@ def create_parsers():
         "to incomplete results.")
     merge_search_parser.add_argument(
         "-e", "--strict-search", action="store_true",
-        help="narrow contact search to name fields")
+        help="DEPRECATED use the new query syntax instead")
     merge_search_parser.add_argument(
-        "-t", "--target-contact", "--target", default="",
-        help="search in all fields to find matching target contact")
+        "-t", "--target-contact", "--target", type=parse,
+        help="search for a matching target contact")
     merge_search_parser.add_argument(
-        "-u", "--uid", default="", help="select source contact by uid")
+        "-u", "--uid", type=lambda x: FieldQuery("uid", x),
+        help="DEPRECATED use the new query syntax instead")
     merge_search_parser.add_argument(
-        "-U", "--target-uid", default="", help="select target contact by uid")
+        "-U", "--target-uid", type=lambda x: FieldQuery("uid", x),
+        help="DEPRECATED use -t with the new query syntax instead")
     merge_search_parser.add_argument(
-        "source_search_terms", nargs="*", metavar="source",
-        help="search in all fields to find matching source contact")
+        "source_search_terms", nargs="*", metavar="source", type=parse,
+        default=[],
+        help="search in specified or all fields to find matching source "
+        "contact")
 
     # create subparsers for actions
-    subparsers = parser.add_subparsers(dest="action")
+    subparsers = parser.add_subparsers(dest="action", metavar="SUBCOMMAND")
     list_parser = subparsers.add_parser(
         "list",
         aliases=Actions.get_aliases("list"),
@@ -156,6 +201,13 @@ def create_parsers():
     list_parser.add_argument(
         "-p", "--parsable", action="store_true",
         help="Machine readable format: uid\\tcontact_name\\taddress_book_name")
+    field_argument = FieldsArgument('index', 'name', 'phone', 'email',
+                                    *CarddavObject.get_properties(),
+                                    nested=True)
+    list_parser.add_argument(
+        "-F", "--fields", default=[], type=field_argument,
+        help="Comma separated list of fields to show "
+        "(use -F help for a list of top level fields)")
     show_parser = subparsers.add_parser(
         "show",
         aliases=Actions.get_aliases("show"),
@@ -171,17 +223,6 @@ def create_parsers():
         type=argparse.FileType("w"),
         help="Specify output template file name or use stdout by default")
     subparsers.add_parser("template", help="print an empty yaml template")
-    export_parser = subparsers.add_parser(
-        "export",
-        aliases=Actions.get_aliases("export"),
-        parents=[default_addressbook_parser, default_search_parser,
-                 sort_parser],
-        description="DEPRECATED (an alias for 'show --format=yaml')",
-        help="DEPRECATED (an alias for 'show --format=yaml')")
-    export_parser.add_argument(
-        "-o", "--output-file", default=sys.stdout,
-        type=argparse.FileType("w"),
-        help="Specify output template file name or use stdout by default")
     birthdays_parser = subparsers.add_parser(
         "birthdays",
         aliases=Actions.get_aliases("birthdays"),
@@ -229,13 +270,6 @@ def create_parsers():
     post_address_parser.add_argument(
         "-p", "--parsable", action="store_true",
         help="Machine readable format: address\\tname\\ttype")
-    subparsers.add_parser(
-        "source",
-        aliases=Actions.get_aliases("source"),
-        parents=[default_addressbook_parser, default_search_parser,
-                 sort_parser],
-        description="DEPRECATED (an alias for 'edit --format=vcard')",
-        help="DEPRECATED (an alias for 'edit --format=vcard')")
     new_parser = subparsers.add_parser(
         "new",
         aliases=Actions.get_aliases("new"),
@@ -257,6 +291,14 @@ def create_parsers():
     add_email_parser.add_argument(
         "--vcard-version", choices=("3.0", "4.0"), dest='preferred_version',
         help="Select preferred vcard version for new contact")
+    add_email_parser.add_argument(
+        "-H", "--headers", default=["from"],
+        type=lambda x: [y.lower() for y in x.split(",")],
+        help="Extract contacts from the given comma separated header fields. "
+        "`all` searches all headers.")
+    add_email_parser.add_argument(
+        "--skip-already-added", action="store_true",
+        help="Skip already added email addresses")
     subparsers.add_parser(
         "merge",
         aliases=Actions.get_aliases("merge"),
@@ -317,18 +359,17 @@ def create_parsers():
     # errors in the config file).  The config file will still be parsed before
     # the full command line is parsed so errors in the config file might be
     # reported before command line syntax errors.
-    first_parser.print_help = parser.print_help
+    first_parser.print_help = parser.print_help  # type: ignore
 
     return first_parser, parser
 
 
-def parse_args(argv):
+def parse_args(argv: List[str]) -> Tuple[argparse.Namespace, Config]:
     """Parse the command line arguments and return the namespace that was
     creates by argparse.ArgumentParser.parse_args().
 
-    :param list(str) argv: the command line arguments
+    :param argv: the command line arguments
     :returns: the namespace parsed from the command line
-    :rtype: argparse.Namespace
     """
     first_parser, parser = create_parsers()
     # Parese the command line with the first argument parser.  It will handle
@@ -344,24 +385,28 @@ def parse_args(argv):
         logging.basicConfig(level=logging.DEBUG)
 
     # Create the config instance.
-    config = Config(args.config)
-    logging.debug("Finished parsing config=%s", vars(config))
+    try:
+        config = Config(args.config)
+    except ConfigError as err:
+        parser.exit(3, "Error in config file: {}\n".format(err))
+    except OSError as err:
+        parser.exit(3, "Error reading config file: {}\n".format(err))
+    logger.debug("Finished parsing config=%s", vars(config))
 
     # Check the log level again and merge the value from the command line with
     # the config file.
     if ("debug" in args and args.debug) or config.debug:
         logging.basicConfig(level=logging.DEBUG)
-    logging.debug("first args=%s", args)
-    logging.debug("remainder=%s", remainder)
+    logger.debug("first args=%s", args)
 
     # Set the default command from the config file if none was given on the
     # command line.
     if not remainder or remainder[0] not in Actions.get_all():
         if config.default_action is None:
-            exit("Missing subcommand on command line or default action "
-                 "parameter in config.")
+            parser.error("Missing subcommand on command line or default action"
+                         " parameter in config.")
         remainder.insert(0, config.default_action)
-        logging.debug("updated remainder=%s", remainder)
+        logger.debug("updated remainder=%s", remainder)
 
     # Save the last option that needs to be carried from the first parser run
     # to the second.
@@ -373,7 +418,7 @@ def parse_args(argv):
 
     # Restore settings that are left from the first parser run.
     args.skip_unparsable = skip
-    logging.debug("second args=%s", args)
+    logger.debug("second args=%s", args)
 
     # An integrity check for some options.
     if "uid" in args and args.uid and (
@@ -382,47 +427,55 @@ def parse_args(argv):
         # If an uid was given we require that no search terms where given.
         parser.error("You can not give arbitrary search terms and --uid at the"
                      " same time.")
+    if "target_uid" in args and args.target_uid and args.target_contact:
+        parser.error("You can not give arbitrary target search terms and "
+                     "--target-uid at the same time.")
+    # Deprecation workaround
+    if "strict_search" in args and args.strict_search:
+        logger.error("Deprecated option --strict-search, use the new query "
+                     "syntax instead.")
+        if "search_terms" in args:
+            args.search_terms = [NameQuery(t.get_term()) for t in
+                                 args.search_terms]
+        if "source_search_terms" in args:
+            args.source_search_terms = [NameQuery(t.get_term()) for t in
+                                        args.source_search_terms]
+        if "taget_search_terms" in args:
+            args.taget_search_terms = [NameQuery(t.get_term()) for t in
+                                       args.taget_search_terms]
 
-    # Normalize all deprecated subcommands and emit warnings.
-    if args.action == "export":
-        logging.warning("Deprecated subcommand: use 'show --format=yaml'.")
-        args.action = "show"
-        args.format = "yaml"
-    elif args.action == "source":
-        logging.warning("Deprecated subcommand: use 'edit --format=vcard'.")
-        args.action = "edit"
-        args.format = "vcard"
+    # Build conjunctive queries.  If uid was given the list of search terms
+    # will be empty.  If no uid was given it will be None.
+    if "source_search_terms" in args:
+        args.source_search_terms = AndQuery.reduce(args.source_search_terms,
+                                                   args.uid)
+    if "search_terms" in args:
+        args.search_terms = AndQuery.reduce(args.search_terms, args.uid)
+    if "target_contact" in args:
+        # Only one of target_contact or target_uid can be set.
+        args.target_contact = args.target_contact or args.target_uid \
+            or AnyQuery()
+    # Remove uid values from the args Namespace.  They have been merged into
+    # the search terms above.
+    if "uid" in args:
+        del args.uid
+    if "target_uid" in args:
+        del args.target_uid
 
     return args, config
 
 
-def merge_args_into_config(args, config):
+def merge_args_into_config(args: argparse.Namespace, config: Config) -> Config:
     """Merge the parsed arguments from argparse into the config object.
 
     :param args: the parsed command line arguments
-    :type args: argparse.Namespace
     :param config: the parsed config file
-    :type config: config.Config
     :returns: the merged config object
-    :rtype: config.Config
-
     """
-    merge = {'general': ['debug'],
-             'contact table': ['reverse', 'group_by_addressbook', 'display',
-                               'sort'],
-             'vcard': ['search_in_source_files', 'skip_unparsable',
-                       'preferred_version'],
-             }
-    merge = {k1: {k2: getattr(args, k2)
-                  for k2 in v1 if k2 in args and getattr(args, k2) is not None}
-             for k1, v1 in merge.items()}
-    logging.debug('Merging in %s', merge)
-    config.merge(merge)
-    logging.debug('Merged: %s', vars(config))
-
+    config.merge_args(args)
     # Now we can savely initialize the address books as all command line
     # options have been incorporated into the config object.
-    config.load_address_books()
+    config.init_address_books()
     # If the user could but did not specify address books on the command line
     # it means they want to use all address books in that place.
     if "addressbook" in args and not args.addressbook:
@@ -432,12 +485,11 @@ def merge_args_into_config(args, config):
     return config
 
 
-def init(argv):
+def init(argv: List[str]) -> Tuple[argparse.Namespace, Config]:
     """Initialize khard by parsing the command line and reading the config file
 
-    :param list(str) argv: the command line arguments
+    :param argv: the command line arguments
     :returns: the parsed command line and the fully initialized config
-    :rtype: (argparse.Namespace, Config)
     """
     args, conf = parse_args(argv)
 
@@ -447,4 +499,7 @@ def init(argv):
         # example: "ls" --> "list"
         args.action = Actions.get_action(args.action)
 
-    return args, merge_args_into_config(args, conf)
+    try:
+        return args, merge_args_into_config(args, conf)
+    except ConfigError as err:
+        sys.exit(str(err))
